#!/tmp/.ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZpErLZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZperl/bin/perl
use strict;
use PureMessage::Manager::Request qw($REQUEST);
use PureMessage::Manager::Response::ServerGroups qw(:all);
use PureMessage::Manager::Widget;
use PureMessage::Manager::Lang;
use PureMessage::ServerGroup qw (:all);
use PureMessage::RPC::Client; # client lib

my $r = $REQUEST->get_response( 'PureMessage::Manager::Response::ServerGroups',
                                'stderr_to_stdout' =>1 );

$r->add_help(text => "Help", tab => "servgrp", page => "mgr_servgrp_over" );

$r->title('services_index_title');

my $grp = get_groups();
my $hosts = get_hosts();
$r->addPaneBox(content => text('no_hosts_or_groups')) unless $hosts;
my $rpc;

if ($hosts) {
    foreach my $group (sort keys %$grp) {
        generate_group($group, $grp->{$group});
    }
}

$r->send();

sub s_icon {
        my ($state,$host,$service) = @_;
        my $url = "service.cgi?service=$service&host=$host";
        my $status = $state
                     ? "running"
                     : $state ne "" ? "stopped" : "unknown";
        return button({ type=>'image',
                        icon=>$service."_".$status, 
                        text=>$status, 
                        href=>$url }, style=>"toolbar");
}

sub s_host_icon {
        my ($d,$name,$r_h) = @_;
        my $h = $r_h->{$name};
        if ($d) {
                return button({ type=>'image', icon=>'key', 
                                alt =>"Authentication key is found",
                                href=>"/ServerGroups/server.cgi?alias=$name"});
        } 
        else {
                return button({ type=>'text',
                                href=>"/ServerGroups/server.cgi?alias=$name"});
        }
}

sub generate_group {
        my $group_name = shift;
        my $group_hash = shift;
        my $group_icon = button({type=>'image', icon=>'hosts',alt=>'hosts'});
        my $title = $group_name =~ /\w/ ? "<a href=\"group.cgi?name=$group_name&load=1\">$group_name</a>&nbsp; &nbsp;" : "";
        my $SND = <<EOF;
        <table width='100%'><tr>
            <td>$group_icon</td>
            <td width="100%" valign="top">$title $group_hash->{description}</td>
        </tr></table>
EOF
        
        my $host_grid = $r->addGrid(
                                    name => $group_name,
                                    columns=>6,
                                    title => $SND,
                                    action=>'index.cgi',
                                    data => sub { get_hosts_for_group($group_name, @_) },
                                    ddata => [
                                                { 
                                                  width=>"3%",
                                                  description=>'',
                                                  extra=>'nowrap',
                                                }, 
                                                { 
                                                  width=>"25%",
                                                  description=>text('host_name_alias'),
                                                  extra=>' nowrap="nowrap" ',
                                                }, 
                                                { 
                                                  width=>"18%",
                                                  description=>'Milter',
                                                  extra=>'nowrap',
                                                }, 
                                                { 
                                                  width=>"18%",
                                                  description=>'SMTP',
                                                  extra=>'nowrap',
                                                }, 
                                                { 
                                                  width=>"18%",
                                                  description=>'HTTP (Manager)',
                                                  extra=>'nowrap',
                                                }, 
                                                { 
                                                  width=>"18%",
                                                  description=>'HTTP (RPC/UI)',
                                                  extra=>'nowrap',
                                                }, 
                                             ],
                                   );
}

sub get_hosts_for_group {
    my $group_name = shift;
    my ($s, $e) = @_;
    my $raw_data;
    unless ($rpc) {
        $rpc = PureMessage::RPC::Client->new($hosts);
        $rpc->call("Services.snapshot");
    }
    unless ($r->{group_data}) {
        $r->{group_data} = {};
        $rpc->wait();
    }
    unless ( $r->{group_data}{$group_name} ) {
        my $grp_hosts = get_hosts($group_name);
        # if we can't get any hosts then there isn't much use trying to get
        # information on them
        if ($rpc) {
            $raw_data = $rpc->data();
            foreach my $host (sort keys %$grp_hosts) {
                next unless $hosts->{$host};
                my @host_data;
                push(@host_data, s_host_icon($grp_hosts->{$host}{auth},$host,$grp_hosts),
                                 button({ type=>'text', href=>"/ServerGroups/server.cgi?alias=$host",
                                          text=>$host
                                       }),
                    );
                if (my $err = $rpc->error($host)) {
                    push(@host_data, 
                         button({type=>"image", icon=>"warning",
                                text=>$err}, style=>"toolbar"), 
                         undef, undef);
                } else {
                    my $d = $raw_data->{$host}{data}[0];
                    push(@host_data, s_icon($d->{milter},$host,'milter'),
                                     s_icon($d->{smtp},$host,'smtp'),
                                     s_icon($d->{http},$host,'http'),
                                     s_icon($d->{httpd},$host,'httpd'),
                        );
                }
                push(@{$r->{group_data}{$group_name}},\@host_data);
            }
        }
    }
    if ( defined $s ) {
        return $r->{group_data}{$group_name};
    }
    else {
        return scalar @{$r->{group_data}{$group_name}} 
            if ref($r->{group_data}{$group_name}) eq "ARRAY";
        return 0;
    }
}
