package PureMessage::RPC::Result;
use Text::CSV_XS;
use strict;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my %self = ();
    $self{serializer} = Text::CSV_XS->new({binary=>1});
    init(\%self);
    return bless( \%self, $class);
}

sub parse {
    my $self = shift;
    my $chunk = shift;
    my $is_body = shift;
    $self->{content} = 1 if $is_body;
    my @data = split (/^/, $self->{incomplete_chunk} . $chunk);
    $self->{incomplete_chunk} = "";
    foreach my $record ( @data ) {
        if ( ! $self->{content} and $record =~ /^\r?\n$/ ) {
            $self->{content} = 1;
            next;
        }
        elsif ( $record =~ /\n/ ) {
            if ( $self->{content} ) {
                $self->decode($record);
            }
            else {
		# this is the basic headers parser for `make test`
		# in a real life Net::HTTP in RPC::Client does this for us
                my ( $k, $v ) = split (/:\s+/, $record);
                $v =~ s/\r\n//g;
                $self->{headers}{$k} = $v;
            }
        }
        else {
            $self->{incomplete_chunk} .= $record;
            #print $self->{incomplete_chunk}."\n";
        }
        if ( $record =~ /\cD/ ) {
            # EOF
            $self->{eof} = 1;
        }
    }
}

sub content_type {
    my $self = shift;
    return $self->{headers}{'Content-Type'} || "unknown";
}

sub decode {
    my $self = shift;
    my $record = shift;
    if ( $self->content_type eq "text/csv" ) {
        my $status = $self->{serializer}->parse($record);
        my @columns = $self->{serializer}->fields();
        if ( scalar @{$self->{meta_data}} > 0 ) {
            # structure is known - store the data
            my %row;
            my $i = 0;
            foreach ( @{$self->{meta_data}} ) {
                $row{$_} = $columns[$i];
                $i ++;
            }
            push @{$self->{data}}, \%row;
        }
        else {
            # this must be a first record -- retrieve metadata
            $self->{meta_data} = \@columns;
        }
    }
    else {
        # raw data
        $self->{data}[0] .= $record;
    }
}

sub init {
    my $self = shift;
    $self->{data} = undef;
    $self->{incomplete_chunk} = "";
    $self->{headers} = {};
    $self->{code} = undef;
    $self->{msg} = undef;
    $self->{content} = 0;
    $self->{meta_data} = [];
    $self->{eof} = 0;
    return 1;
}

sub error {
    my $self = shift;
    if ( scalar(@{$self->{meta_data}}) == 1 and $self->{meta_data}[0] eq 'ERROR' ) {
        return join("", map { $_->{ERROR} } @{$self->{data}} );
    }
    return 0;
}

sub _undef {
    my $self = shift;
    if ( scalar(@{$self->{meta_data}}) == 1 and $self->{meta_data}[0] eq 'UNDEF' ) {
        return 1;
    }
    return 0;
}

sub data {
    my $self = shift;
    return undef if $self->error() or $self->_undef();
    if ( scalar(@{$self->{meta_data}}) == 1 and $self->{meta_data}[0] eq 'DATA' ) {
        # SCALAR
        return join("", map { $_->{DATA} } @{$self->{data}} );
    }
    elsif ( $self->{meta_data} ) {
	return undef unless ref($self->{data}) eq "ARRAY";
        if ( wantarray() ) {
            # ARRAY
            return @{$self->{data}};
        }
        if ( scalar @{$self->{data}} > 1 ) {
            # ARRAY ref
            return $self->{data};
        }
        else {
            # HASH ref
            return $self->{data}[0];
        }
    }
    else {
        # SCALAR (not marchalled data)
        return $self->{data}[0] if scalar @{$self->{data}} > 0;
    }
}

1;
